/* Copyright (C) 2013 by Bundesanstalt für Gewässerkunde
 * Software engineering by Intevation GmbH
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */

package org.dive4elements.river.exports.process;

import org.apache.log4j.Logger;
import org.jfree.data.xy.XYSeries;

import org.dive4elements.artifactdatabase.state.ArtifactAndFacet;
import org.dive4elements.artifacts.CallContext;
import org.dive4elements.river.artifacts.model.FacetTypes;
import org.dive4elements.river.exports.DiagramGenerator;
import org.dive4elements.river.exports.StyledSeriesBuilder;
import org.dive4elements.river.jfree.StyledXYSeries;
import org.dive4elements.river.themes.ThemeDocument;

import org.dive4elements.river.artifacts.model.FlowVelocityData;

public class FlowVelocityProcessor extends DefaultProcessor {

    private final static Logger logger =
            Logger.getLogger(FlowVelocityProcessor.class);

    public static final String I18N_AXIS_LABEL =
        "chart.flow_velocity.section.yaxis.label";
    public static final String I18N_AXIS_LABEL_DEFAULT =
        "Geschwindigkeit v [m/s]";

    @Override
    public void doOut(
            DiagramGenerator generator,
            ArtifactAndFacet bundle,
            ThemeDocument    theme,
            boolean          visible) {
        CallContext context = generator.getCallContext();
        XYSeries series = new StyledXYSeries(bundle.getFacetDescription(),
                theme);
        FlowVelocityData data = (FlowVelocityData) bundle.getData(context);
        if (data == null) {
            /* Check was there in the old generator */
            logger.error("Flow velocity data is null. Bad facet.");
            return;
        }
        String facetName = bundle.getFacetName();
        double [][] points;

        if (facetName.equals(FacetTypes.FLOW_VELOCITY_TOTALCHANNEL) ||
                facetName.equals(FacetTypes.FLOW_VELOCITY_TOTALCHANNEL_FILTERED)) {
            points = data.getTotalChannelPoints();
        } else if (facetName.equals(FacetTypes.FLOW_VELOCITY_MAINCHANNEL) ||
                facetName.equals(FacetTypes.FLOW_VELOCITY_MAINCHANNEL_FILTERED)) {
            points = data.getMainChannelPoints(); // I hate facets!
        } else {
            logger.error("Unknown facet name: " + facetName);
            return;
        }
        StyledSeriesBuilder.addPoints(series, points, true);

        generator.addAxisSeries(series, axisName, visible);
    }

    @Override
    public boolean canHandle(String facettype) {
        return facettype.equals(FacetTypes.FLOW_VELOCITY_MAINCHANNEL_FILTERED) ||
            facettype.equals(FacetTypes.FLOW_VELOCITY_MAINCHANNEL) ||
            facettype.equals(FacetTypes.FLOW_VELOCITY_TOTALCHANNEL_FILTERED) ||
            facettype.equals(FacetTypes.FLOW_VELOCITY_TOTALCHANNEL);
    }

    @Override
    public String getAxisLabel(DiagramGenerator generator) {
        return generator.msg(
                I18N_AXIS_LABEL,
                I18N_AXIS_LABEL_DEFAULT);
    }
}
